/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by VIA Networking Technologies, Inc.
 * 4. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY VIA NETWORKING TECHNOLOGIES, INC. AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL 
 * VIA NETWORKING TECHNOLOGIES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * File: if_getb_desc.h
 *
 * Purpose: TD/RD ring structures definition
 *
 * Author: Ryan Fu
 *
 * Date: Jul 22, 2003
 *
 *
 */

#ifndef __IF_GETB_DESC_H__
#define __IF_GETB_DESC_H__

#include <dev/getb/if_getb_cfg.h>

#define CB_RX_BUF_SIZE      2048UL

#define CB_MAX_RD_NUM       512
#define CB_MAX_TD_NUM       256

#define CB_INIT_RD_NUM_3119 128
#define CB_INIT_TD_NUM_3119 64

#define CB_INIT_RD_NUM      128
#define CB_INIT_TD_NUM      64

#define CB_TD_RING_NUM      4
#define CB_MAX_SEG_PER_PKT  7

#define CB_MAX_SEGMENT      7

#define CB_MAX_TX_ABORT_RETRY   3



/*---------------------  Export Types  ------------------------------*/
/*
typedef struct __rdesc0 {
    U16 wRSR;
    U16 f14Length : 14;
    U16 f1Resv    : 1;
    U16 f1Owner   : 1;
}
__attribute__ ((__packed__))
RDESC0, *PRDESC0;

typedef struct __rdesc1 {
    U16   wPQTAG;
    U8    byCSM;
    U8    byIPKT;
}
__attribute__ ((__packed__))
RDESC1, *PRDESC1;

typedef struct __rx_desc {
    RDESC0  rdesc0;
    RDESC1  rdesc1;
    U32     dwBufAddrLo;
    U16     wBufAddrHi;
    U16     f15BufLen  : 15;
    U16     f1IntCtlEn : 1;
}
__attribute__ ((__packed__))
RX_DESC, * PRX_DESC;

typedef struct _tdesc0 {
    volatile    U16    wTSR;
    volatile    U16    f14PktSize   : 14;
    volatile    U16    f1Reserved   : 1;
    volatile    U16    f1Owner      : 1;
}
__attribute__ ((__packed__))
TDESC0, *PTDESC0;

typedef struct _pqinf {
    U16     f12VID       : 12;
    U16     f1CFI        : 1;
    U16     f3Priority   : 3;
}
__attribute__ ((__packed__))
PQINF, *PPQINF;

typedef struct _tdesc1 {
    PQINF   pqinf;
    U8      byTCR;
    U8      f2TCPLS      : 2;
    U8      f2Reserved   : 2;
    U8      f4CMDZ       : 4;
}
__attribute__ ((__packed__))
TDESC1, *PTDESC1;

typedef struct _td_buf {
    U32     dwBufAddrLo;
    U16     wBufAddrHi;
    U16     f14BufSize  :   14;
    U16     f1Reserved  :   1;
    U16     f1Queue     :   1;
}
__attribute__ ((__packed__))
TD_BUF, *PTD_BUF;

typedef struct _tx_desc {
    TDESC0  tdesc0;
    TDESC1  tdesc1;
    TD_BUF  aTdBufs[CB_MAX_SEGMENT];
}
__attribute__ ((__packed__))
TX_DESC, *PTX_DESC;
*/

/* DMA descriptor structures */

#define OWNED_BY_NIC         0x80000000
#define RXDESC_I             0x8000
#define TXDESC_Q             0x8000
/*
 * Each TX DMA descriptor has a control and status word, and 7
 * fragment address/length words. If a transmitted packet spans
 * more than 7 fragments, it has to be coalesced.
 */

typedef struct _tx_frag {
	uint32_t        dwBufAddrLo;
	uint16_t        wBufAddrHi;
	uint16_t        wBufSize;
} TD_BUF, *PTD_BUF;

/*
 * The high bit in the buflen field of fragment #0 has special meaning.
 * Normally, the chip requires the driver to issue a TX poll command
 * for every packet that gets put in the TX DMA queue. Sometimes though,
 * the driver might want to queue up several packets at once and just
 * issue one transmit command to have all of them processed. In order
 * to obtain this behavior, the special 'queue' bit must be set.
 */
typedef struct _tx_desc {
	volatile uint32_t  td0sts;
	volatile uint32_t  td1ctl;
        TD_BUF             aTdBufs[CB_MAX_SEGMENT];
} TX_DESC, *PTX_DESC;

#define TDSTS_PKTSIZE      0x3FFF0000    /* TX packet size */
#define TSR_TERR           0x00008000
#define TSR_FDX            0x00004000
#define TSR_GMII           0x00002000
#define TSR_LNKFL          0x00001000
#define TSR_SHDN           0x00000400
#define TSR_CRS            0x00000200
#define TSR_CDH            0x00000100
#define TSR_ABT            0x00000080
#define TSR_OWT            0x00000040
#define TSR_OWC            0x00000020
#define TSR_COLS           0x00000010
#define TSR_NCR3           0x00000008
#define TSR_NCR2           0x00000004
#define TSR_NCR1           0x00000002
#define TSR_NCR0           0x00000001

#define TCR_CMDZ           0xF0000000
#define TCPLS_BITS         0x03000000
#define TCPLS_NORMAL       0x03000000
#define TCPLS_START        0x02000000
#define TCPLS_END          0x01000000
#define TCPLS_MED          0x00000000

#define TCR_TIC            0x00800000
#define TCR_PIC            0x00400000
#define TCR_VETAG          0x00200000
#define TCR_IPCK           0x00100000
#define TCR_UDPCK          0x00080000
#define TCR_TCPCK          0x00040000
#define TCR_JMBO           0x00020000
#define TCR_CRC            0x00010000
#define VGE_PRIO           0x0000E000   /* VLAN prio bits */
#define TCR_CFI            0x00001000   /* VLAN CFI bit */
#define TCR_VLANID         0x00000FFF   /* VLAN ID */



/* Receive DMA descriptors have a single fragment pointer. */

typedef struct _rx_desc {
	volatile uint32_t  rd0sts;
	volatile uint32_t  rd1ctl;
	volatile uint32_t  dwBufAddrLo;
	volatile uint16_t  wBufAddrHi;
	volatile uint16_t  wBufLen;
} RX_DESC, *PRX_DESC;

/*
 * Like the TX descriptor, the high bit in the buflen field in the
 * RX descriptor has special meaning. This bit controls whether or
 * not interrupts are generated for this descriptor.
 */
#define RDSTS_BUFSIZ        0x3FFF0000    /* received frame len */
#define RSR_RXOK            0x00008000
#define RSR_PFT             0x00004000
#define RSR_MAR             0x00002000
#define RSR_BAR             0x00001000
#define RSR_PHY             0x00000800
#define RSR_VTAG            0x00000400
#define RSR_STP             0x00000200
#define RSR_EDP             0x00000100
#define RSR_DETAG           0x00000080
#define RSR_SNTAG           0x00000040
#define RSR_RXER            0x00000020
#define RSR_RL              0x00000010
#define RSR_CE              0x00000008
#define RSR_FAE             0x00000004
#define RSR_CRC             0x00000002
#define RSR_VIDM            0x00000001

#define CSM_IPOK            0x00400000
#define CSM_TUPOK           0x00200000
#define CSM_FRAG            0x00100000
#define CSM_IPKT            0x00040000
#define CSM_TCPKT           0x00020000
#define CSM_UDPKT           0x00010000
#define RDCTL_PQTAG         0x0000FFFF

typedef struct _rhine_rd_info {
    struct mbuf*        mbuf;
    void*               mbuf_dma;
} GETB_RD_INFO, *PGETB_RD_INFO;

static __inline PGETB_RD_INFO alloc_rd_info(void)
{
    PGETB_RD_INFO   ptr;
    if ((ptr=malloc(sizeof(GETB_RD_INFO), M_DEVBUF, M_NOWAIT))==NULL)
        return NULL;
    else {
        bzero(ptr, sizeof(GETB_RD_INFO));
        return ptr;
    }
}

typedef struct _rhine_td_info {
    struct mbuf*        mbuf;
} GETB_TD_INFO, *PGETB_TD_INFO;

/*
typedef enum {
    OWNED_BY_HOST=0,
    OWNED_BY_NIC=1
} GETB_OWNER_TYPE, *PGETB_OWNER_TYPE;
*/

/*---------------------  Export Macros ------------------------------*/

/*---------------------  Export Classes  ----------------------------*/

/*---------------------  Export Variables  --------------------------*/

/*---------------------  Export Functions  --------------------------*/


#endif /* __IF_GETB_DESC_H__ */

